

##########################################################
##########################################################
##########################################################
#
#       NONBCG program
#
#       Analysis of     non-vaccinated (non-BCG) data
#
#       Input data in   infile
#
##########################################################
##########################################################
##########################################################


Version <- "NONBCG (version 3.1, December 2004)"

obj <- objects()

nonbcg <- function()
{

if (is.na(match("RndSeed",obj))) RndSeed <- runif(1,0,100)

set.seed(RndSeed)

if (is.na(match("outfile",obj))) outfile <- "TBMixOut"

outfilelog <- paste(outfile,".log",sep="")
outfile1 <- paste(outfile,"1.txt",sep="")
outfile2 <- paste(outfile,"2.txt",sep="")
outfile3 <- paste(outfile,"3.txt",sep="")

############################################################
sink(file=outfilelog,append=F)
 cat(paste("Program",Version),"\n")
 cat(date(),"\n")
sink()
Sys.sleep(1)
############################################################


############################################################
sink(file=outfilelog,append=T)
 cat("Preparatory steps","\n")
 cat(" ...Load and process data","\n")
sink()
Sys.sleep(1)
############################################################

if (is.na(match("header",obj))) header <- F

num.cova <-  length(freq.column);
ds <- read.table(file=infile,header=header)
ds <- as.matrix(ds)

if ( dim(ds)[1]==1 & dim(ds)[2]==31 )
{
warning("Frequencies should be entered as column data!")
ds <- t(ds)
}

if (is.na(match("freq.column",obj)))    freq.column <- seq(1,dim(ds)[2])

if ( length(freq.column)>1  & is.na(match("group.names",obj))) group.names <- paste("Group",seq(1,length(freq.column)))
if ( length(freq.column)==1 & is.na(match("group.names",obj)))    group.names <- ""


ind.num <- dim(ds)[1]-1
ind <- seq(1,ind.num);
freqzero <- ds[,freq.column];
freq <-  ds[2:(ind.num+1),freq.column];

if (is.na(match("ind.brk",obj)))        {
ind.brk <- seq(1.5,by=1,length=dim(ds)[1]-2)
}

if (length(freq.column)==1)     {
freqzero <- matrix(freqzero,ind.num+1,1)
freq  <- matrix(freq ,ind.num,1)
}


if (length(freq.column)>1)      {
n.total <- apply(freqzero,2,sum)
}
else    {
n.total <- sum(freqzero)
}


if (num.cova>1)
freqall <- rbind(freqzero,n.total);

if (num.cova==1)
freqall <- matrix(c(freqzero,n.total),ncol=1);

if (ind.num==30)
pred.int <- matrix(c(1,2,3,7,8,12,13,17,18,22,23,27,28,30),7,2,byrow=T);

if (is.na(match("ind.names",obj)))      {
if (ind.num==30)  ind.names <- paste(seq(1,30),"mm",sep="")
else ind.names <- paste("ind-",seq(1,ind.num),sep="")
ind0 <- ifelse(ind.num==30,"0mm","ind-0")
}

ind.names0 <- c(ind0,ind.names)
ind.namesntot <- c(ind.names0,"total")

if (ind.num==30) ind.brk <- seq(1.5,29.5,by=1);

if (ind.num==30)        {
len1 <- dim(pred.int)[1];
freq1 <- matrix(0,len1,num.cova);
freq11 <- matrix(0,len1,num.cova);
ind.names1 <- character(0);
for (jj in 1:len1)
ind.names1[jj] <- paste(pred.int[jj,1], "to", pred.int[jj,2], "mm");
}

for (j in 1:num.cova)   {
if (ind.num==30)        {
for (jj in 1:len1)
freq1[jj,j] <- sum( freq[pred.int[jj,1]:pred.int[jj,2],j]);
}
}


#       normalizing likelihood (will be subtracted from likelihood)
l0 <- sum( apply(freqzero,2,function(x) sum(x*log(x/sum(x)),na.rm=T) ) )
LLnorm <- l0


############################################################
sink(file=outfilelog,append=T)
 cat(" ...Set tuning parameters for Metropolis Sampler","\n")
sink()
Sys.sleep(1)
############################################################


if (is.na(match("MS.run",obj))) MS.run <- T
if (is.na(match("MS.results",obj))) MS.results <- T
if (is.na(match("MS.check",obj))) MS.check <- F
if (is.na(match("MS.graph",obj))) MS.graph <- c(T,F,F,F,T)

if (is.na(match("MS.graph.page",obj))) {
pg <- ceiling(sqrt(num.cova))
if (pg*(pg-1)>=num.cova) MS.graph.page <- c(pg-1,pg) else MS.graph.page <- c(pg,pg)
}

if (is.na(match("MS.size",obj))) MS.size <- c(length(freq.column)*500,5000,1)
if (length(MS.size)==2) MS.size[3] <- 1

if (is.na(match("MS.screen",obj))) MS.screen <- c(5,6, 1,2,3,4 )
if (is.na(match("MS.screen.dig",obj))) MS.screen.dig <- 3

if (is.na(match("MS.adj",obj))) MS.adj <- 200

MS.names <- c(
"TB.qnt1",
"TB.qnt2",
"CR.qnt1",
"CR.qnt2",
"p.TB",
"p.zero"
)


############################################################
sink(file=outfilelog,append=T)
cat(" ...Load information from former analysis","\n")
sink()
Sys.sleep(1)
############################################################

if (!is.na(match("TBmix",obj))) {
fit <- TBmix$fit
pred.table <- TBmix$pred.table
p.TB.summary <- TBmix$p.TB.summary
p.zero.summary <- TBmix$p.zero.summary
TB.qnt1.summary <- TBmix$TB.qnt1.summary
TB.qnt2.summary <- TBmix$TB.qnt2.summary
CR.qnt1.summary <- TBmix$CR.qnt1.summary
CR.qnt2.summary <- TBmix$CR.qnt2.summary
mpe0 <- TBmix$mpe
LastValues <- TBmix$LastValues
}
else    {
fit <- NA
pred.table <- NA
p.TB.summary <- NA
p.zero.summary <-  NA
TB.qnt1.summary <- NA
TB.qnt2.summary <- NA
CR.qnt1.summary <- NA
CR.qnt2.summary <- NA
mpe0 <- NA
}

############################################################
sink(file=outfilelog,append=T)
cat(" ...Set values for parameter constraints","\n")
sink()
Sys.sleep(1)
############################################################

# intervals
if (is.na(match("TB.qnt1.int",obj)))  TB.qnt1.int <- c(0,40)
if (is.na(match("TB.qnt2.int",obj)))  TB.qnt2.int <- c(0,40)
if (is.na(match("CR.qnt1.int",obj)))  CR.qnt1.int <- c(0,40)
if (is.na(match("CR.qnt2.int",obj)))  CR.qnt2.int <- c(0,40)

if (is.na(match("p.TB.int",obj)))       p.TB.int <-  c(0,1)
if (is.na(match("p.zero.int",obj)))     p.zero.int <-  c(0,1)

# diff
if (is.na(match("TB.qnt1.diff",obj))) TB.qnt1.diff <- 0
if (is.na(match("TB.qnt2.diff",obj))) TB.qnt2.diff <- 0
if (is.na(match("CR.qnt1.diff",obj))) CR.qnt1.diff <- 0
if (is.na(match("CR.qnt2.diff",obj))) CR.qnt2.diff <- 0

# ratio
if (is.na(match("TB.qnt1.ratio",obj))) TB.qnt1.ratio <- 1
if (is.na(match("TB.qnt2.ratio",obj))) TB.qnt2.ratio <- 1
if (is.na(match("CR.qnt1.ratio",obj))) CR.qnt1.ratio <- 1
if (is.na(match("CR.qnt2.ratio",obj))) CR.qnt2.ratio <- 1

if (is.na(match("qnt1TB",obj))) qnt1TB <- 0.50
if (is.na(match("qnt2TB",obj))) qnt2TB <- 0.95
if (is.na(match("qnt1CR",obj))) qnt1CR <- 0.50
if (is.na(match("qnt2CR",obj))) qnt2CR <- 0.95

TB.x1 <- qnorm(qnt1TB);
TB.x2 <- qnorm(qnt2TB);
CR.x1 <- qnorm(qnt1CR);
CR.x2 <- qnorm(qnt2CR);

############################################################
sink(file=outfilelog,append=T)
 cat(" ...Load functions","\n")
sink()
Sys.sleep(1)
############################################################

#       MakeGroupNames
#       MakeGroupIndices
#       MakeGroupSelect
#       CompareGroupsDiff
#       CompareGroupsRatio
#       MakeCheck
#       PrepareCheck
#       Checks
#       loglik
#       prior0
#       trans0
#       mixture


MakeGroupNames <- function(group.names)
{
groups.vector <- group.names[[1]]
if (length(group.names)>1)  {
for (j in seq(2,length(group.names)) ) {
groups.vector <- as.vector(t( outer(groups.vector,group.names[[j]],paste) ))
}
}
return(group.names=groups.vector)
}

MakeGroupIndices <- function(groups.indices)
{
groups.vector <- groups.indices[[length(groups.indices)]]
for (j in ( (length(groups.indices)-1)):1)       {
groups.vector <-  rep((groups.indices[[j]]-1)*length(groups.vector),each=length(groups.vector))+groups.vector
}
}

MakeGroupSelect <- function(groups.sizes)
{
dlist <- list()

for (j in 1:length(groups.sizes))
{
dlist[[j]] <- seq(1,groups.sizes[j])
}

groups.select <- matrix(NA,prod(groups.sizes),length(groups.sizes) )

for (i in 1:length(groups.sizes))
{
for (j in 1:groups.sizes[i])
{
ddlist <- dlist
ddlist[[i]] <- rep(j,groups.sizes[i])
mmm <- MakeGroupIndices( ddlist)
groups.select[mmm,i] <- j
}
}
return(groups.select)
}

CompareGroupsDiff <- function(groups.select,groups.sizes,crit)
{
sss <- seq(1,prod(groups.sizes))
Alist <- list()

for (k in 1:length(groups.sizes))
        {

if (any(crit[[k]] !=0))
                {
A <- numeric(0)

for (i in 1:(groups.sizes[k]-1) )
{
if (length(crit[[k]])==1)
 jup <- groups.sizes[k]
 else
 jup <- i+1
for (j in (i+1):jup   )
{

e1 <- groups.select[,k]==i
e2 <- groups.select[,k]==j
AA1 <- sss[e1]
AA2 <- sss[e2]
cr <- unlist(crit[[k]])
AA <- cbind(AA1,AA2,matrix(cr,sum(e1),length(cr),byrow=T))
A <- rbind(A,AA)
}
}
Alist[[k]] <- A
}
else Alist[[k]] <- NA
}

return(Alist)
}

CompareGroupsRatio <- function(groups.select,groups.sizes,crit)
{
sss <- seq(1,prod(groups.sizes))

Alist <- list()

for (k in 1:length(groups.sizes))
        {


if ( any(crit[[k]] !=1))
                {
A <- numeric(0)

for (i in 1:(groups.sizes[k]-1) )
{
if (length(crit[[k]])==1)
 jup <- groups.sizes[k]
 else
 jup <- i+1
for (j in (i+1):jup   )
{

e1 <- groups.select[,k]==i
e2 <- groups.select[,k]==j
AA1 <- sss[e1]
AA2 <- sss[e2]
cr <- unlist(crit[[k]])
AA <- cbind(AA1,AA2,matrix(cr,sum(e1),length(cr),byrow=T))
A <- rbind(A,AA)
}
}
Alist[[k]] <- A
}
else Alist[[k]] <- NA

}
return(Alist)
}

MakeCheck <- function(para,CG,comp)
{

if (!is.list(CG)) CG <- list(CG)

TFvec <- rep(NA,length(CG))

for (j in 1:length(CG))
{
CG1 <- CG[[j]]

if ( all(is.na(CG1)) ) TF <- T
else
{

if (comp=="diff")
{
out <- para[CG1[,2]]-para[CG1[,1]]
if (ncol(CG1)==3) TF <- abs(out) < abs(CG1[,3])
if (ncol(CG1)==4) TF <- out < CG1[,4] & out > CG1[,3]
}

if (comp=="ratio")
{
out <- para[CG1[,2]]/para[CG1[,1]]
if (ncol(CG1)==3) TF <- abs(log(out)) < log(CG1[,3])
if (ncol(CG1)==4) TF <- out < CG1[,4] & out > CG1[,3]
}

TF <- all(TF)
}

TFvec[j] <- TF

}
return(all(TFvec))
}


PrepareCheck <- function(group.names,crit,comp)
{

if (is.list(group.names))
{
groups.sizes <- unlist(lapply(group.names[2:length(group.names)],length))

if (length(crit)>1)
 {
 GS <- MakeGroupSelect(groups.sizes)

 if (comp=="diff") rt <- list(Check=any(unlist(crit)!=0),comp,CheckValues=CompareGroupsDiff(GS,groups.sizes,crit))
 if (comp=="ratio") rt <- list(Check=any(unlist(crit)!=1),comp,CheckValues=CompareGroupsRatio(GS,groups.sizes,crit))
 }

if (length(crit)==1)
{
 if (crit==0 & comp=="diff"   ) rt <- list(Check=F,comp,CheckValues=0)
 if (crit>0  & comp=="diff"    ) rt <- list(Check=T,comp,CheckValues=crit)
 if (crit==1 & comp=="ratio"   ) rt <- list(Check=F,comp,CheckValues=1)
 if (crit>1  & comp=="ratio"    ) rt <- list(Check=T,comp,CheckValues=crit)
}
}  # end of list

else
{
groups.sizes <- num.cova
if (length(group.names)==1)
        rt <- list(Check=F,CheckValues=0)

if (length(group.names)>1 )
{
if (length(crit)==1)
{
 if (crit==0 & comp=="diff"   ) rt <- list(Check=F,comp,CheckValues=0)
 if (crit>0  & comp=="diff"    ) rt <- list(Check=T,comp,CheckValues=crit)
 if (crit==1 & comp=="ratio"   ) rt <- list(Check=F,comp,CheckValues=1)
 if (crit>1  & comp=="ratio"    ) rt <- list(Check=T,comp,CheckValues=crit)
}
if (length(crit)>1)
 {
 #GS <- matrix(seq(1:num.cova),num.cova,1)
 #if (comp=="diff") rt <- list(Check=any(unlist(crit)!=0),comp,CheckValues=CompareGroupsDiff(GS,groups.sizes,crit))
 #if (comp=="ratio") rt <- list(Check=any(unlist(crit)!=1),comp,CheckValues=CompareGroupsRatio(GS,groups.sizes,crit))
 rt <- list( Check=T, comp,
 CheckValues=cbind( seq(1:(num.cova-1)), seq(2,num.cova), rep(crit[1],num.cova-1), rep(crit[2],num.cova-1) ) )
 }
}
}       # end of vector

return(rt)
}


Checks <- function(para,PC)
{
if (!PC[[1]])
        rt <- T

if (PC[[1]])
{

if (length(PC[[3]])==1)
{
 if (PC[[2]]=="diff")
 mm <- max(para)-min(para)
 if (PC[[2]]=="ratio")
 mm <- max(para)/min(para)
 rt <- mm<=PC[[3]]
}

if (length(PC[[3]])>1)
{
 rt <- MakeCheck(para,PC[[3]],PC[[2]])
}
}
return(rt)
}

#       FUNCTIONS:  LOG-LIKELIHOOD, LOG-PRIOR

loglik <- function(theta,x)
{
theta[,1] <- theta[groups.indices[,1],1]
theta[,2] <- theta[groups.indices[,2],2]
theta[,3] <- theta[groups.indices[,3],3]
theta[,4] <- theta[groups.indices[,4],4]

p1      <- theta[,5];
p0      <- theta[,6];
TB.qnt1  <- theta[,1];
TB.qnt2  <- theta[,2];
CR.qnt1  <- theta[,3];
CR.qnt2  <- theta[,4];

ll <- -1e300;

cond2a <- Checks(TB.qnt1,TB.qnt1.diff.PC)
cond3a <- Checks(TB.qnt2,TB.qnt2.diff.PC)
cond4a <- Checks(CR.qnt1,CR.qnt1.diff.PC)
cond5a <- Checks(CR.qnt2,CR.qnt2.diff.PC)

cond2b <- Checks(TB.qnt1,TB.qnt1.ratio.PC)
cond3b <- Checks(TB.qnt2,TB.qnt2.ratio.PC)
cond4b <- Checks(CR.qnt1,CR.qnt1.ratio.PC)
cond5b <- Checks(CR.qnt2,CR.qnt2.ratio.PC)

cond10 <- all(TB.qnt1<TB.qnt2)
cond11 <- all(CR.qnt1<CR.qnt2)
cond12 <- all(CR.qnt1<TB.qnt1)
cond13 <- all(CR.qnt2<TB.qnt2)
cond20 <- all(p1+p0<1)

cond <- c(cond2a,cond3a,cond4a,cond5a,cond2b,cond3b,cond4b,cond5b, cond10,cond11,cond12,cond13,cond20)

if ( all(cond) )      {
ll <- 0
for (jjj in 1:num.cova) {
if (distTB=="Wb")
 dist1 <- Wb.dsc(ind.brk,TB.qnt1[jjj],TB.qnt2[jjj],c(qnt1TB,qnt2TB) );
if (distTB=="LN")       {
  sigma  <- (log(TB.qnt2[jjj])-log(TB.qnt1[jjj]))/(TB.x2-TB.x1);
  mu <- log(TB.qnt1[jjj])-sigma*TB.x1;
  dist1 <- LN.dsc(ind.brk,mu,sigma,0);
}
if (distTB=="N")        {
  sigma  <- (TB.qnt2[jjj]-TB.qnt1[jjj])/(TB.x2-TB.x1);
  mu <- TB.qnt1[jjj]-sigma*TB.x1;
  dist1 <- N.dsc(ind.brk,mu,sigma);
}
if (distCR=="Wb")
  dist2 <- Wb.dsc(ind.brk,CR.qnt1[jjj],CR.qnt2[jjj],c(qnt1CR,qnt2CR) );
if (distCR=="LN")       {
  sigma   <- (log(CR.qnt2[jjj])-log(CR.qnt1[jjj]))/(CR.x2-CR.x1);
  mu <- log(CR.qnt1[jjj])-sigma*CR.x1;
  dist2 <- LN.dsc(ind.brk,mu,sigma,0);
}

dist1[is.na(dist1)] <- 1e-300;
dist2[is.na(dist2)] <- 1e-300;
lladd <- freq[,jjj]*log( (1-p0[jjj]-p1[jjj])*dist2 + p1[jjj]*dist1) ;

if(any(is.na(lladd))) lladd <- -1e300;
ll <- ll+sum(lladd);
}
}
ll <- ll+sum(freqzero[1,]*log(p0));
return(ll-LLnorm);
}



prior0 <- function(theta)       {return(0)}

trans0 <- function(theta,x)
{
theta[,1] <- theta[groups.indices[,1],1]
theta[,2] <- theta[groups.indices[,2],2]
theta[,3] <- theta[groups.indices[,3],3]
theta[,4] <- theta[groups.indices[,4],4]

out <-
list(
TB.qnt1=theta[,1],
TB.qnt2=theta[,2],
CR.qnt1=theta[,3],
CR.qnt2=theta[,4],
p.TB=theta[,5],
p.zero=theta[,6])

return(out)
}

mixture <- function(theta)
{
TB.qnt1 <- theta[1];
TB.qnt2 <- theta[2];
CR.qnt1 <- theta[3];
CR.qnt2 <- theta[4];
p1 <- theta[5];
p0 <- theta[6];
p2 <- 1-p0-p1

if (distTB=="Wb")
 dist1 <- Wb.dsc(ind.brk,TB.qnt1,TB.qnt2,c(qnt1TB,qnt2TB) );
if (distTB=="LN")       {
  sigma   <- (log(TB.qnt2)-log(TB.qnt1))/(TB.x2-TB.x1)
  mu <- log(TB.qnt1)-sigma*TB.x1;
  dist1  <- LN.dsc(ind.brk,mu,sigma,0);
}
if (distTB=="N")        {
  sigma  <- (TB.qnt2-TB.qnt1)/(TB.x2-TB.x1);
  mu <- TB.qnt1-sigma*TB.x1;
  dist1 <- N.dsc(ind.brk,mu,sigma);
}
if (distCR=="Wb")
 dist2 <- Wb.dsc(ind.brk,CR.qnt1,CR.qnt2,c(qnt1CR,qnt2CR) );
if (distCR=="LN")       {
  sigma   <- (log(CR.qnt2)-log(CR.qnt1))/(CR.x2-CR.x1)
  mu <- log(CR.qnt1)-sigma*CR.x1;
  dist2  <- LN.dsc(ind.brk,mu,sigma,0);
}


p1f1 <- p1*dist1;
p2f2 <- p2*dist2;

mix  <- c(p0,p1f1+p2f2)

mix[is.na(mix)] <- 1e-300;
p1f1[is.na(p1f1)] <- 1e-300;
p2f2[is.na(p2f2)] <- 1e-300;

return(list(mix=mix,p1TB=p1f1,p2CR=p2f2));
}



############################################################
sink(file=outfilelog,append=T)
 cat(" ...Create grouping information","\n")
sink()
Sys.sleep(1)
############################################################


groups.labels <- group.names

if (is.vector(group.names) & !is.list(group.names) )     {

groups.labels <- group.names
groups.numbers <- 1
groups.indices <- matrix(NA,length(group.names),10)

if ( all(TB.qnt1.diff==0) & all(TB.qnt1.ratio==1))
groups.indices[,1] <- rep(1,length(group.names))
else
groups.indices[,1] <- seq(1,length(group.names))

if (all(TB.qnt2.diff==0) & all(TB.qnt2.ratio==1))
groups.indices[,2] <- rep(1,length(group.names))
else
groups.indices[,2] <- seq(1,length(group.names))

if (all(CR.qnt1.diff==0) & all(CR.qnt1.ratio==1))
groups.indices[,3] <- rep(1,length(group.names))
else
groups.indices[,3] <- seq(1,length(group.names))

if (all(CR.qnt2.diff==0) & all(CR.qnt2.ratio==1))
groups.indices[,4] <- rep(1,length(group.names))
else
groups.indices[,4] <- seq(1,length(group.names))

}

if (is.list(group.names))       {

if ( length(group.names) != length(group.names[[1]]) + 1 )
stop("Invalid input for group.names")

group.names1 <- group.names[2:length(group.names)]
groups.labels <- MakeGroupNames(group.names1)


groups.numbers <- length(group.names)-1
groups.sizes <- numeric(0)
for (j in 2:(groups.numbers+1) )    {
groups.sizes[j-1] <- length(group.names[[j]])
}

groups.indices <- matrix(NA,length(freq.column),10)

if (length(TB.qnt1.diff)==1) TB.qnt1.diff <- rep(TB.qnt1.diff,groups.numbers)
if (length(TB.qnt2.diff)==1) TB.qnt2.diff <- rep(TB.qnt2.diff,groups.numbers)
if (length(CR.qnt1.diff)==1) CR.qnt1.diff <- rep(CR.qnt1.diff,groups.numbers)
if (length(CR.qnt2.diff)==1) CR.qnt2.diff <- rep(CR.qnt2.diff,groups.numbers)

if (length(TB.qnt1.ratio)==1) TB.qnt1.ratio <- rep(TB.qnt1.ratio,groups.numbers)
if (length(TB.qnt2.ratio)==1) TB.qnt2.ratio <- rep(TB.qnt2.ratio,groups.numbers)
if (length(CR.qnt1.ratio)==1) CR.qnt1.ratio <- rep(CR.qnt1.ratio,groups.numbers)
if (length(CR.qnt2.ratio)==1) CR.qnt2.ratio <- rep(CR.qnt2.ratio,groups.numbers)

iii <- list()
for (j in 1:groups.numbers) {
if ( all(TB.qnt1.diff[[j]]==0) & all(TB.qnt1.ratio[[j]]==1) ) iii[[j]] <- rep(1,groups.sizes[j])
else iii[[j]] <- seq(1,groups.sizes[j])
}
groups.indices[,1] <- MakeGroupIndices(iii)

iii <- list()
for (j in 1:groups.numbers) {
if ( all(TB.qnt2.diff[[j]]==0) & all(TB.qnt2.ratio[[j]]==1) ) iii[[j]] <- rep(1,groups.sizes[j])
else iii[[j]] <- seq(1,groups.sizes[j])
}
groups.indices[,2] <- MakeGroupIndices(iii)

iii <- list()
for (j in 1:groups.numbers) {
if ( all(CR.qnt1.diff[[j]]==0) & all(CR.qnt1.ratio[[j]]==1) ) iii[[j]] <- rep(1,groups.sizes[j])
else iii[[j]] <- seq(1,groups.sizes[j])
}
groups.indices[,3] <- MakeGroupIndices(iii)

iii <- list()
for (j in 1:groups.numbers) {
if ( all(CR.qnt2.diff[[j]]==0) & all(CR.qnt2.ratio[[j]]==1) ) iii[[j]] <- rep(1,groups.sizes[j])
else iii[[j]] <- seq(1,groups.sizes[j])
}
groups.indices[,4] <- MakeGroupIndices(iii)

}


############################################################
sink(file=outfilelog,append=T)
 cat(" ...Prepare checks for component distributions","\n")
sink()
Sys.sleep(1)
############################################################

TB.qnt1.diff.PC <- PrepareCheck(group.names,TB.qnt1.diff,"diff")
TB.qnt2.diff.PC <- PrepareCheck(group.names,TB.qnt2.diff,"diff")
CR.qnt1.diff.PC <- PrepareCheck(group.names,CR.qnt1.diff,"diff")
CR.qnt2.diff.PC <- PrepareCheck(group.names,CR.qnt2.diff,"diff")

TB.qnt1.ratio.PC <- PrepareCheck(group.names,TB.qnt1.ratio,"ratio")
TB.qnt2.ratio.PC <- PrepareCheck(group.names,TB.qnt2.ratio,"ratio")
CR.qnt1.ratio.PC <- PrepareCheck(group.names,CR.qnt1.ratio,"ratio")
CR.qnt2.ratio.PC <- PrepareCheck(group.names,CR.qnt2.ratio,"ratio")


############################################################
sink(file=outfilelog,append=T)
 cat(" ...Create initial values for model parameters","\n")
sink()
Sys.sleep(1)
############################################################



if ( !is.na(match("MS.InitialValues",obj))  )
theta00 <- MS.InitialValues


if ( is.na(match("MS.InitialValues",obj)) )
{

p.zero.init <- freqzero[1,]/n.total

if (is.na(match("p.TB.init",obj)))      {

if (num.cova>1) {
if (ind.num==30)
 p.TB.init <- apply(freqzero[11:31,],2,sum)/n.total
 if (ind.num!=30)
 p.TB.init <- (1-p.zero.init)/2
}

if (num.cova==1) {
if (ind.num==30)
 p.TB.init <- sum(freqzero[11:31,])/n.total
if (ind.num!=30)
 p.TB.init <- (1-p.zero.init)/2
}
}


if (is.na(match("TB.qnt1.init",obj)))   TB.qnt1.init <- 18
if (is.na(match("TB.qnt2.init",obj)))   TB.qnt2.init <- 30
if (is.na(match("CR.qnt1.init",obj)))   CR.qnt1.init <- 2
if (is.na(match("CR.qnt2.init",obj)))   CR.qnt2.init <- 5

theta00 <- matrix(NA,num.cova,6)
theta00[,5] <- p.TB.init
theta00[,6] <- p.zero.init
theta00[,1] <- TB.qnt1.init;
theta00[,2] <- TB.qnt2.init;
theta00[,3] <- CR.qnt1.init;
theta00[,4] <- CR.qnt2.init;

}

theta00[-groups.indices[,1],1] <- NA
theta00[-groups.indices[,2],2] <- NA
theta00[-groups.indices[,3],3] <- NA
theta00[-groups.indices[,4],4] <- NA

theta0 <- theta00
dimnames(theta0) <- list(NULL,NULL)

dimnames(theta00) <- list(groups.labels,c(
"TBqnt1","TBqnt2",
"CRqnt1","CRqnt2",
"pTB","pzero"
))


cat("Initial Values","\n")
print(theta00,digits=3)
llinits <- loglik(theta00,0)
cat("\n")
cat("Log-likelihood at initial values:",llinits,"\n")

if (llinits <= -1e300)
stop("Numerical problems: try different starting values for parameters")


############################################################
sink(file=outfilelog,append=T)
 cat(" ...Check initial model parameters","\n")
sink()
Sys.sleep(1)
############################################################

#       check initial parameter matrix
th0 <- theta0
th0[,1] <- theta0[groups.indices[,1],1]
th0[,2] <- theta0[groups.indices[,2],2]
th0[,3] <- theta0[groups.indices[,3],3]
th0[,4] <- theta0[groups.indices[,4],4]
p1      <- th0[,5];
p0      <- th0[,6];
TB.qnt1  <- th0[,1];
TB.qnt2  <- th0[,2];
CR.qnt1  <- th0[,3];
CR.qnt2  <- th0[,4];

cond1a <- Checks(TB.qnt1,TB.qnt1.diff.PC)
cond2a <- Checks(TB.qnt2,TB.qnt2.diff.PC)
cond3a <- Checks(CR.qnt1,CR.qnt1.diff.PC)
cond4a <- Checks(CR.qnt2,CR.qnt2.diff.PC)

cond1b <- Checks(TB.qnt1,TB.qnt1.ratio.PC)
cond2b <- Checks(TB.qnt2,TB.qnt2.ratio.PC)
cond3b <- Checks(CR.qnt1,CR.qnt1.ratio.PC)
cond4b <- Checks(CR.qnt2,CR.qnt2.ratio.PC)


cond <- c(
cond1a,cond2a,cond3a,cond4a,cond1b,cond2b,cond3b,cond4b
)

if (!all(cond))
{
if (!cond1a) stop("Initial values for TB.qnt1 not consistent with TB.qnt1.diff")
if (!cond2a) stop("Initial values for TB.qnt2 not consistent with TB.qnt2.diff")
if (!cond3a) stop("Initial values for CR.qnt1 not consistent with CR.qnt1.diff")
if (!cond4a) stop("Initial values for CR.qnt2 not consistent with CR.qnt2.diff")

if (!cond1b) stop("Initial values for TB.qnt1 not consistent with TB.qnt1.ratio")
if (!cond2b) stop("Initial values for TB.qnt2 not consistent with TB.qnt2.ratio")
if (!cond3b) stop("Initial values for CR.qnt1 not consistent with CR.qnt1.ratio")
if (!cond4b) stop("Initial values for CR.qnt2 not consistent with CR.qnt2.ratio")
}


if ( !all(TB.qnt1<TB.qnt2) ) stop("Initial values TB.qnt1<TB.qnt2 violation")
if ( !all(CR.qnt1<CR.qnt2) ) stop("Initial values CR.qnt1<CR.qnt2 violation")
if ( !all(CR.qnt1<TB.qnt1) ) stop("Initial values CR.qnt1<TB.qnt1 violation")
if ( !all(CR.qnt2<TB.qnt2) ) stop("Initial values CR.qnt2<TB.qnt2 violation")

if ( any(p1+p0>=1) ) stop("p0+p1>1 violation")




############################################################
sink(file=outfilelog,append=T)
 cat(" ...Boundaries for parameters","\n")
sink()
Sys.sleep(1)
############################################################

MS.low <- NA*theta0
MS.high <- NA*theta0

MS.low[,1] <- TB.qnt1.int[1]
MS.low[,2] <- TB.qnt2.int[1]
MS.low[,3] <- CR.qnt1.int[1]
MS.low[,4] <- CR.qnt2.int[1]
MS.low[,5] <- p.TB.int[1]
MS.low[,6] <- p.zero.int[1]

MS.high[,1] <- TB.qnt1.int[2]
MS.high[,2] <- TB.qnt2.int[2]
MS.high[,3] <- CR.qnt1.int[2]
MS.high[,4] <- CR.qnt2.int[2]
MS.high[,5] <- p.TB.int[2]
MS.high[,6] <- p.zero.int[2]


MS.high[,5:6] <- 1

if (is.na(match("MS.scale",obj)))
MS.scale <- matrix( c(0.1,0.1, 0.05,0.05,  0.01, 0.01 ), nrow(theta0),ncol(theta0), byrow=T)

if (is.na(match("MS.block",obj)))
MS.block <- matrix(c(1,2,3,4, 5,6),nrow(theta0),ncol(theta0),byrow=T)


############################################################
sink(file=outfilelog,append=T)
 cat(" ...Create labels","\n")
sink()
Sys.sleep(1)
############################################################


para.names <- c(
paste("TB(",100*qnt1TB,"%) ",groups.labels,sep=""),
paste("TB(",100*qnt2TB,"%) ",groups.labels,sep=""),
paste("CR(",100*qnt1CR,"%) ",groups.labels,sep=""),
paste("CR(",100*qnt2CR,"%) ",groups.labels,sep=""),
paste("p.TB",groups.labels),
paste("p.zero",groups.labels),
);

out.str <- c("mean",      "st.dev",    "2.5%",      "50%",       "97.5%" )

dimnames(freqall) <- list(ind.namesntot,groups.labels)



if (MS.run)     {

############################################################
sink(file=outfilelog,append=T)
 cat("MS.run: Start Metropolis Sampler","\n")
sink()
Sys.sleep(1)
############################################################

cat("\n",paste("Program",Version),"\n");
cat("\n","Metropolis Sampling (MS) algorithm ...","\n")

fit <- Metropolis(data=0,loglik="loglik",logprior="prior0",gibbs="gibbs0",trans="trans0",
theta0=theta0,MS.adj=MS.adj,MS.scale=MS.scale,MS.block=MS.block,
MS.low=MS.low,MS.high=MS.high,MS.size=MS.size,MS.names=MS.names,MS.screen=MS.screen,MS.screen.dig=MS.screen.dig)


}

if (MS.results)         {

############################################################
sink(file=outfilelog,append=T)
 cat("MS.results: Processing results","\n")
sink()
Sys.sleep(1)
############################################################


############################################################
sink(file=outfilelog,append=T)
 cat(" ...Loading sampled values","\n")
sink()
Sys.sleep(1)
############################################################

TB.qnt1 <- fit$Sample[[1]]
TB.qnt2 <- fit$Sample[[2]]
CR.qnt1 <- fit$Sample[[3]]
CR.qnt2 <- fit$Sample[[4]]

p.TB <- fit$Sample[[5]]
p.zero <- fit$Sample[[6]]

sink(file=outfile1,append=F);

############################################################
sink(file=outfilelog,append=T)
 cat(" ...Basic input for analysis","\n")
sink()
Sys.sleep(1)
############################################################


cat(date(),"\n","\n");
cat(paste("Program",Version),"\n");
cat("Mixture Analysis of induration data: TB infections (TB), crossreactions (CR)","\n","\n");

cat("File specifications","\n")
cat(" Data:         ",infile,"\n")
cat(" Output files: ",outfile,"\n","\n")

if (!is.na(match("RndSeed",obj)))
cat("Random seed at start:", RndSeed,"\n","\n")

if (is.list(group.names))
cat("Data grouped by:",group.names[[1]],"\n","\n")


if (ind.num != 30)      {
cat("Grouped induration data with intervals defined by cut-offs:","\n")
cat(ind.brk)
cat("\n","\n")
}

cat("Frequencies","\n");
print(freqall);


cat("\n");
cat("Metropolis Sampler:","\n");
cat(" burn-in:           ",MS.size[1],"\n");
cat(" sample size:       ",MS.size[2],"\n");
cat(" thinning:          ",MS.size[3],"\n");
cat(" no. of iterations: ",MS.size[1]+MS.size[2]*MS.size[3],"\n","\n");

cat("Distribution of TB infections:  ", distTB,"\n");
cat("  parameters defined by two quantiles:  ", qnt1TB,qnt2TB,"\n","\n");

cat("Distribution of crossreactions: ", distCR,"\n");
cat("  parameters defined by two quantiles:  ", qnt1CR,qnt2CR,"\n","\n");


############################################################
sink(file=outfilelog,append=T)
 cat(" ...Parameter constraints","\n")
sink()
Sys.sleep(1)
############################################################


if (TB.qnt1.diff.PC[[1]])
{
cat("\n")
cat("Differences for TB distribution (1st quantile):","\n")
if (length(TB.qnt1.diff)==1)            cat(TB.qnt1.diff,"\n")
if (length(TB.qnt1.diff)>1)
{
for (j in 1:length(TB.qnt1.diff))       cat(group.names[[1]][j],TB.qnt1.diff[[j]],"\n")
}
}

if (TB.qnt1.ratio.PC[[1]])
{
cat("\n")
cat("Ratios for TB distribution (1st quantile):","\n")
if (length(TB.qnt1.ratio)==1)            cat(TB.qnt1.ratio,"\n")
if (length(TB.qnt1.ratio)>1)
{
for (j in 1:length(TB.qnt1.ratio))       cat(group.names[[1]][j],TB.qnt1.ratio[[j]],"\n")
}
}


if (TB.qnt2.diff.PC[[1]])
{
cat("\n")
cat("Differences for TB distribution (2nd quantile):","\n")
if (length(TB.qnt2.diff)==1)            cat(TB.qnt2.diff,"\n")
if (length(TB.qnt2.diff)>1)
{
for (j in 1:length(TB.qnt2.diff))       cat(group.names[[1]][j],TB.qnt2.diff[[j]],"\n")
}
}

if (TB.qnt2.ratio.PC[[1]])
{
cat("\n")
cat("Ratios for TB distribution (2nd quantile):","\n")
if (length(TB.qnt2.ratio)==1)            cat(TB.qnt2.ratio,"\n")
if (length(TB.qnt2.ratio)>1)
{
for (j in 1:length(TB.qnt2.ratio))       cat(group.names[[1]][j],TB.qnt2.ratio[[j]],"\n")
}
}




if (CR.qnt1.diff.PC[[1]])
{
cat("\n")
cat("Parameter constraints: Differences for CR distribution (1st quantile)","\n")
if (length(CR.qnt1.diff)==1)            cat(CR.qnt1.diff,"\n")
if (length(CR.qnt1.diff)>1)
{
for (j in 1:length(CR.qnt1.diff))       cat(group.names[[1]][j],CR.qnt1.diff[[j]],"\n")
}
}

if (CR.qnt1.ratio.PC[[1]])
{
cat("\n")
cat("Parameter constraints: Ratios for CR distribution (1st quantile)","\n")
if (length(CR.qnt1.ratio)==1)            cat(CR.qnt1.ratio,"\n")
if (length(CR.qnt1.ratio)>1)
{
for (j in 1:length(CR.qnt1.ratio))       cat(group.names[[1]][j],CR.qnt1.ratio[[j]],"\n")
}
}


if (CR.qnt2.diff.PC[[1]])
{
cat("\n")
cat("Parameter constraints: Differences for CR distribution (2nd quantile)","\n")
if (length(CR.qnt2.diff)==1)            cat(CR.qnt2.diff,"\n")
if (length(CR.qnt2.diff)>1)
{
for (j in 1:length(CR.qnt2.diff))       cat(group.names[[1]][j],CR.qnt2.diff[[j]],"\n")
}
}

if (CR.qnt2.ratio.PC[[1]])
{
cat("\n")
cat("Parameter constraints: Ratios for CR distribution (2nd quantile)","\n")
if (length(CR.qnt2.ratio)==1)            cat(CR.qnt2.ratio,"\n")
if (length(CR.qnt2.ratio)>1)
{
for (j in 1:length(CR.qnt2.ratio))       cat(group.names[[1]][j],CR.qnt2.ratio[[j]],"\n")
}
}


cat("\n")
cat("Maximum log-likelihood (from simulation): ", -fit$Deviance$Dhat.mpe/2,"\n","\n");

############################################################
sink(file=outfilelog,append=T)
 cat(" ...Posterior summaries","\n")
sink()
Sys.sleep(1)
############################################################


cat("Parameter estimates","\n")

cat("\n","Prevalence of TB infections (p.TB)","\n")
p.TB.summary <- MSoutput(p.TB,c(0.025,0.50,0.975),groups.labels)
print(p.TB.summary,digits=3)

cat("\n","Prevalence of no reaction (p.zero)","\n")
p.zero.summary <- MSoutput(p.zero,c(0.025,0.50,0.975),groups.labels)
print(p.zero.summary,digits=3)

cat("\n","TB distribution (1st quantile, TB.qnt1)","\n")
TB.qnt1.summary <- MSoutput(TB.qnt1,c(0.025,0.50,0.975),groups.labels)
print(TB.qnt1.summary,digits=3)

cat("\n","TB distribution (2nd quantile, TB.qnt2)","\n")
TB.qnt2.summary <- MSoutput(TB.qnt2,c(0.025,0.50,0.975),groups.labels)
print(TB.qnt2.summary,digits=3)

cat("\n","CR distribution (1st quantile, CR.qnt1)","\n")
CR.qnt1.summary <- MSoutput(CR.qnt1,c(0.025,0.50,0.975),groups.labels)
print(CR.qnt1.summary,digits=3)

cat("\n","CR distribution (2nd quantile, CR.qnt2)","\n")
CR.qnt2.summary <- MSoutput(CR.qnt2,c(0.025,0.50,0.975),groups.labels)
print(CR.qnt2.summary,digits=3)


############################################################
sink(file=outfilelog,append=T)
 cat(" ...Maximum posterior estimates ","\n")
sink()
Sys.sleep(1)
############################################################


mpe00 <- fit$Estimates$mpe

mpe0 <- theta0*NA
mpe0[,5] <- mpe00[[5]]
mpe0[,6] <- mpe00[[6]]

mpe0[unique(groups.indices[,1]),1] <- mpe00[[1]]
mpe0[unique(groups.indices[,2]),2] <- mpe00[[2]]
mpe0[unique(groups.indices[,3]),3] <- mpe00[[3]]
mpe0[unique(groups.indices[,4]),4] <- mpe00[[4]]

mpe0[,1] <- mpe0[groups.indices[,1],1]
mpe0[,2] <- mpe0[groups.indices[,2],2]
mpe0[,3] <- mpe0[groups.indices[,3],3]
mpe0[,4] <- mpe0[groups.indices[,4],4]

#       summary of posterior parameter estimates
cat("\n");
cat("Maximum posterior estimates","\n");
dimnames(mpe0) <- list(groups.labels,MS.names)
print(mpe0,digits=3);


LastValues <- fit$Output$theta.last
LastValues[,1] <- LastValues[groups.indices[,1],1]
LastValues[,2] <- LastValues[groups.indices[,2],2]
LastValues[,3] <- LastValues[groups.indices[,3],3]
LastValues[,4] <- LastValues[groups.indices[,4],4]

############################################################
sink(file=outfilelog,append=T)
 cat(" ...Total prevalences","\n")
sink()
Sys.sleep(1)
############################################################

p.TB.total <- matrix(0,nrow(p.TB),1);
p.zero.total <- p.TB.total;

for (i in 1:nrow(p.TB) )    {
for (j in 1:num.cova)       {
p.TB.total[i] <- p.TB.total[i]+n.total[j]*p.TB[i,j]
p.zero.total[i]  <- p.zero.total[i]+n.total[j]*p.zero[i,j]
}
}

p.TB.total <- p.TB.total/sum(n.total);
p.zero.total <- p.zero.total/sum(n.total);

p.TB.total.sum <- MSoutput(p.TB.total,c(0.025,0.50,0.975),"p.TB");
p.zero.total.sum <- MSoutput(p.zero.total,c(0.025,0.50,0.975),"p.zero");

cat("\n","Total prevalence of TB infections","\n");
print(p.TB.total.sum,digits=3);
cat("\n","Total prevalence of zero reaction","\n");
print(p.zero.total.sum,digits=3);


cat("\n","\n")

cat("Initial Values","\n")
print(theta00,digits=3)
llinits <- loglik(theta00,0)
cat("\n")
cat("Log-likelihood at initial values:",llinits,"\n")

sink()

############################################################
sink(file=outfilelog,append=T)
 cat(" ...Fitted frequencies","\n")
sink()
Sys.sleep(1)
############################################################

sink(file=outfile2,append=F);

cat("\n")

mixobs <- matrix(0,ind.num,num.cova);
p1f1obs <- mixobs;
p2f2obs <- mixobs;

for (j in 1:num.cova)   {
cat("\n",groups.labels[j],"\n");
ffreq <- freq[,j];
theta <- numeric(0);
theta <- mpe0[j,c(1,2,3,4,5,6)]
dist <- mixture(theta);
mix <- dist$mix;
n <- sum(freqzero[,j]);
distout <- cbind(freqzero[,j],n*dist$mix,c(0,n*dist$p1TB),c(0,n*dist$p2CR));
dimnames(distout) <- list( ind.names0,c("obs","mixture","p1*TB", "p2*CR"));
print(round(distout,3));
mixobs[,j] <- mix[2:(ind.num+1)]/sum(mix[2:(ind.num+1)])*sum(freq[,j]);
}

cat("\n")

sink();
} # end of analysis section


if (MS.check)           {
#       posterior predictive checks

############################################################
sink(file=outfilelog,append=T)
 cat("MS.check: Predictive model checks","\n")
sink()
Sys.sleep(1)
############################################################


TB.qnt1 <- fit$Sample[[1]]
TB.qnt2 <- fit$Sample[[2]]
CR.qnt1 <- fit$Sample[[3]]
CR.qnt2 <- fit$Sample[[4]]

p.TB <- fit$Sample[[5]]
p.zero <- fit$Sample[[6]]

sink(file=outfile3,append=F);

pred.table <- matrix(0,ind.num,num.cova)

cat("\n","\n")
cat(
"Model checks: observed vs. predicted frequencies (difference)","\n");
cat(
"-------------------------------------------------------------","\n","\n")


for (j in 1:num.cova)   {
sink();

cat("\n","posterior predictive model check: ",groups.labels[j] );

sink(file=outfile3,append=T);

cat(groups.labels[j],"\n");
ffreq <- freq[,j];
predmat <- matrix(0,nrow(p.TB),length(ind));

if (ind.num==30)
predmat1 <- matrix(0,nrow(p.TB),len1);

for (i in 1:nrow(p.TB) ) {
ix <- seq(j,by=num.cova,length=6);

para <- c(TB.qnt1[i,j],TB.qnt2[i,j],CR.qnt1[i,j],CR.qnt2[i,j],p.TB[i,j],p.zero[i,j])

p <- mixture(para);
p <- p$mix;
p <- p[2:(ind.num+1)];
p <- p/sum(p);
p <- c(0,cumsum(p));
h <- hist(runif(sum(ffreq),0,1),breaks=p,plot=F);
predmat[i,] <- ffreq-h$counts;

if (ind.num==30) {
for (jj in 1:len1)  {
predmat1[i,jj] <- freq1[jj,j]-sum(h$counts[pred.int[jj,1]:pred.int[jj,2]]);
}
}

}       # end of i loop

#       differences: observed - predicted
obs.pred <- MSoutput(predmat,c(0.025,0.50,0.975));
obs.pred <- cbind(ffreq,ffreq-obs.pred[,1],obs.pred);

if (ind.num==30)        {
obs.pred1 <- MSoutput(predmat1,c(0.025,0.50,0.975));
obs.pred1 <- cbind(freq1[,j],freq1[,j]-obs.pred1[,1],obs.pred1);
}

dimnames(obs.pred) <- list(ind.names,c("obs","pred",paste("diff-",out.str,sep="")));

if (ind.num==30)        {
dimnames(obs.pred1) <- list(ind.names1,c("observed","predicted",out.str));
}
print(round(obs.pred,1))

pred.low <- which(obs.pred[,5]>0);
pred.high <- which(obs.pred[,7]<0);
pred.table[pred.low,j] <-  -1
pred.table[pred.high,j] <- 1

#print(pred.table)

cat("\n");
cat("Predictive failures","\n");
cat(" predictions too small: ",ind.names[pred.low],"\n");
cat(" predictions too large: ",ind.names[pred.high],"\n","\n");

if (ind.num==30)        {
print(round(obs.pred1,1));
pred.low <- which(obs.pred1[,5]>0);
pred.high <- which(obs.pred1[,7]<0);
cat("\n");
cat("Predictive failures","\n");
cat(" predictions too small: ",ind.names1[pred.low],"\n");
cat(" predictions too large: ",ind.names1[pred.high],"\n","\n");
}

}       # end of j loop



############################################################
sink(file=outfilelog,append=T)
 cat(" ...Predictive summaries","\n")
sink()
Sys.sleep(1)
############################################################


pred.table <- matrix(pred.table,ind.num,num.cova)
pred.table <- rbind( pred.table, apply(pred.table,2,function(x) sum(abs(x)) ) )
pred.table <- cbind( pred.table, apply(pred.table,1,function(x) sum(abs(x)) ) )

pp <- matrix(pred.table[1:ind.num,1:num.cova])

replace(pp,-1,"-")
replace(pp,0,"")
replace(pp,1,"+")

pred.table[1:ind.num,1:num.cova] <- pp
pred.table <- data.frame(pred.table)


cat("\n","\n")
cat("Summary of predictive checks (+ predictions too large, - predictions too small)")
cat("\n","  ",round(100*pred.table[ind.num+1,num.cova+1]/ind.num/num.cova,1),"% predictive failures","\n","\n")


dimnames(pred.table) <- list(c(ind.names,"total"),c(seq(1,num.cova),"total"))
cat( paste(seq(1,num.cova),": ",groups.labels,sep=""),"\n","\n")
print(pred.table)
cat("\n")

sink();

sink(file=outfile1,append=T)

cat("\n","\n")
cat("Summary of predictive checks (+ predictions too large, - predictions too small)")
cat("\n","  ",round(100*pred.table[ind.num+1,num.cova+1]/ind.num/num.cova,1),"% predictive failures","\n","\n")


dimnames(pred.table) <- list(c(ind.names,"total"),c(seq(1,num.cova),"total"))
cat( paste(seq(1,num.cova),": ",groups.labels,sep=""),"\n","\n")
print(pred.table)
cat("\n")

sink()

}


if (any(MS.graph))   {

############################################################
sink(file=outfilelog,append=T)
 cat("MS.graph: Creating graphs","\n")
sink()
Sys.sleep(1)
############################################################


if (length(MS.graph)==1) MS.graph <- rep(MS.graph,5)
perpage <- MS.graph.page[1]*MS.graph.page[2];

indind <- c(0,ind.brk,30.5)
indind <- (indind[2:length(indind)]+indind[1:(length(indind)-1)])/2

win.graph()

if (MS.size[3]>1) maintext <- paste("Deviance from sampled values (thinning=",MS.size[3],")",sep="")
else maintext="Deviance from sampled values"

plot( fit$LoglikLogpostSample[,3], -2*fit$LoglikLogpostSample[,2],
type="n",ylab="Deviance",xlab="",main=maintext)

e <- fit$LoglikLogpostSample[,3]<=0
lines( fit$LoglikLogpostSample[e,3], -2*fit$LoglikLogpostSample[e,2],lty=2)
e <- fit$LoglikLogpostSample[,3]>=0
lines( fit$LoglikLogpostSample[e,3], -2*fit$LoglikLogpostSample[e,2],lty=1)
abline(v=0,lty=2)
#text(0-MS.size[2]*0.02,min(fit$LoglikLogpostSample[,2]),"burn-in",adj=1)
#text(0+MS.size[2]*0.02,min(fit$LoglikLogpostSample[,2]),"sample",adj=0)


#       MCMC plots


if (MS.graph[1])  {

############################################################
sink(file=outfilelog,append=T)
 cat(" ...MCMC plots","\n")
sink()
Sys.sleep(1)
############################################################

paras <-
cbind(
fit$Sample[[1]],
fit$Sample[[2]],
fit$Sample[[3]],
fit$Sample[[4]],
fit$Sample[[5]],
fit$Sample[[6]])

for (i in 1:length(para.names)) {

if (i%%perpage==1 | perpage==1 )      {
win.graph()
par(mfrow=c(MS.graph.page[1],MS.graph.page[2]));
}

plot(paras[,i],type="l",main=para.names[i],xlab="",ylab="");
}

}      #  MCMC plots (end)

#       SAMPLE HISTOGRAMS

if (MS.graph[2])  {

############################################################
sink(file=outfilelog,append=T)
 cat(" ...Histograms for parameter estimates","\n")
sink()
Sys.sleep(1)
############################################################

paras <-
cbind(
fit$Sample[[1]],
fit$Sample[[2]],
fit$Sample[[3]],
fit$Sample[[4]],
fit$Sample[[5]],
fit$Sample[[6]])

#       SAMPLE HISTOGRAMS
for (i in 1:length(para.names)) {

if (i%%perpage==1 | perpage==1)      {
win.graph()
par(mfrow=c(MS.graph.page[1],MS.graph.page[2]));
}

hist(paras[,i],type="l",main=para.names[i],xlab="",ylab="");
}

}    #   sample histograms (end)


#       PROBABILITY of INFECTION

if (MS.graph[3])    {

############################################################
sink(file=outfilelog,append=T)
 cat(" ...Graphs for probability of infection","\n")
sink()
Sys.sleep(1)
############################################################

mixobs <- matrix(0,ind.num,num.cova);
p1f1obs <- mixobs;
p2f2obs <- mixobs;

for (j in 1:num.cova)   {
theta <- mpe0[j,c(1,2,3,4,5,6)]
dist <- mixture(theta);
mix <- dist$mix;
n <- sum(freqzero[,j]);
mixobs[,j] <-  dist$mix[2:(ind.num+1)]*n;
p1f1obs[,j] <- dist$p1TB*n;
p2f2obs[,j] <- dist$p2CR*n;
}

for (i in 1:num.cova)   {

if (i%%perpage==1 | perpage==1)      {
win.graph()
par(mfrow=c(MS.graph.page[1],MS.graph.page[2]));
}

plot(indind,p1f1obs[,i]/(p1f1obs[,i]+p2f2obs[,i]),type="l",
xlab="induration",ylab="Pr [ TB | ind ]",
main = groups.labels[i],
xlim=c(0,20),ylim=c(0,1));
abline(h=0.50,lty=3);
}

}   #   probability of infection (end)



#       CONFIDENCE intervales (only if number of groups > 1)
if (MS.graph[4])
{

if (num.cova>1)
{

win.graph()
GS <- p.TB.summary
plot(seq(1,num.cova),GS[,4],pch=16,xlab="group",ylab="p.TB",
main="Prevalence of TB infection",ylim=c(min(GS[,3]),max(GS[,5])) )
for (j in 1:num.cova){
lines(c(j,j),GS[j,c(3,5)])
}

win.graph()
GS <- p.zero.summary
plot(seq(1,num.cova),GS[,4],pch=16,xlab="group",ylab="p.zero",
main="Prevalence of zero induration",ylim=c(min(GS[,3]),max(GS[,5])) )
for (j in 1:num.cova){
lines(c(j,j),GS[j,c(3,5)])
}


win.graph()
par(mfrow=c(2,2))

GS <- TB.qnt1.summary
plot(seq(1,num.cova),GS[,4],pch=16,xlab="group",ylab="TB.qnt1",
main="TB.qnt1",ylim=c(min(GS[,3]),max(GS[,5])) )
for (j in 1:num.cova){
lines(c(j,j),GS[j,c(3,5)])
}

GS <- TB.qnt2.summary
plot(seq(1,num.cova),GS[,4],pch=16,xlab="group",ylab="TB.qnt2",
main="TB.qnt2",ylim=c(min(GS[,3]),max(GS[,5])) )
for (j in 1:num.cova){
lines(c(j,j),GS[j,c(3,5)])
}

GS <- CR.qnt1.summary
plot(seq(1,num.cova),GS[,4],pch=16,xlab="group",ylab="CR.qnt1",
main="CR.qnt1",ylim=c(min(GS[,3]),max(GS[,5])) )
for (j in 1:num.cova){
lines(c(j,j),GS[j,c(3,5)])
}

GS <- CR.qnt2.summary
plot(seq(1,num.cova),GS[,4],pch=16,xlab="group",ylab="CR.qnt2",
main="CR.qnt2",ylim=c(min(GS[,3]),max(GS[,5])) )
for (j in 1:num.cova){
lines(c(j,j),GS[j,c(3,5)])
}


}

}      #  confidence intervals (end)



#       HISTOGRAMS and FIT

if (MS.graph[5])
{

############################################################
sink(file=outfilelog,append=T)
 cat(" ...Data histograms and fits","\n")
sink()
Sys.sleep(1)
############################################################

mixobs <- matrix(0,ind.num,num.cova);
p1f1obs <- mixobs;
p2f2obs <- mixobs;

for (j in 1:num.cova)   {
theta <- mpe0[j,c(1,2,3,4,5,6)]
dist <- mixture(theta);
mix <- dist$mix;
n <- sum(freqzero[,j]);
mixobs[,j] <-  dist$mix[2:(ind.num+1)]*n;
p1f1obs[,j] <- dist$p1TB*n;
p2f2obs[,j] <- dist$p2CR*n;
}

if (length(MS.graph.page)==1)
MS.graph.page <- c(MS.graph.page,MS.graph.page)


for (i in 1:num.cova)   {

if (i%%perpage==1 | perpage==1)      {
win.graph()
par(mfrow=c(MS.graph.page[1],MS.graph.page[2]));
}


if ( max(mixobs[,i]) > max(freq[,i]) )  {
plot(indind,mixobs[,i],type="l",ylab="n",
main=groups.labels[i]);
lines(indind,p1f1obs[,i],lty=2);
lines(indind,p2f2obs[,i],lty=2);
points(c(indind,31)-0.5,c(freq[,i],0),type="s");
}

if ( max(mixobs[,i]) < max(freq[,i]) )  {
plot(c(indind,31)-0.5,c(freq[,i],0),type="s",ylab="n",xlab="induration",
main=groups.labels[i]);
lines(indind,mixobs[,i]);
lines(indind,p1f1obs[,i],lty=2);
lines(indind,p2f2obs[,i],lty=2);
}

} # end of i-loop


}     #   end of histograms and fit



}       #  end of MS.graph



Program=paste("Program",Version)

MS.info <- list(
MS.size=MS.size,
MS.block=MS.block,
MS.low=MS.low,
MS.high=MS.high,
MS.adj=MS.adj,
MS.screen=MS.screen,
MS.screen.dig=MS.screen.dig,
MS.names=MS.names,
MS.InitialValues=theta00,
MS.run=MS.run,
MS.results=MS.results,
MS.check=MS.check,
MS.graph=MS.graph,
MS.graph.page=MS.graph.page
)

Data=freqall

files <- c(infile,outfile)
dist <- c(distTB=distTB,distCR=distCR)

constraints <- list(
TB.qnt1.diff=TB.qnt1.diff,
TB.qnt2.diff=TB.qnt2.diff,
CR.qnt1.diff=CR.qnt1.diff,
CR.qnt2.diff=CR.qnt2.diff,
TB.qnt1.ratio=TB.qnt1.ratio,
TB.qnt2.ratio=TB.qnt2.ratio,
CR.qnt1.ratio=CR.qnt1.ratio,
CR.qnt2.ratio=CR.qnt2.ratio
)

############################################################
sink(file=outfilelog,append=T)
 cat("Clearing Memory","\n")
sink()
Sys.sleep(1)
############################################################

############################################################
sink(file=outfilelog,append=T)
 cat(date(),"\n")
sink()
Sys.sleep(1)
############################################################


return(
list(
Program=Program,
fit=fit,
pred.table=pred.table,
MSinfo=MS.info,
constraints=constraints,
files=files,
dist=dist,
Data=Data,
p.TB.summary=p.TB.summary,
p.zero.summary=p.zero.summary,
TB.qnt1.summary=TB.qnt1.summary,
TB.qnt2.summary=TB.qnt2.summary,
CR.qnt1.summary=CR.qnt1.summary,
CR.qnt2.summary=CR.qnt2.summary,
mpe=mpe0,
LastValues=LastValues,
RndSeedAtStart=RndSeed)
)

}


TBmix <- nonbcg()

if (!is.na(match("TB.qnt1.diff",obj))) rm(TB.qnt1.diff )
if (!is.na(match("TB.qnt2.diff",obj))) rm(TB.qnt2.diff )
if (!is.na(match("CR.qnt1.diff",obj))) rm(CR.qnt1.diff )
if (!is.na(match("CR.qnt2.diff",obj))) rm(CR.qnt2.diff )

if (!is.na(match("TB.qnt1.ratio",obj))) rm(TB.qnt1.ratio )
if (!is.na(match("TB.qnt2.ratio",obj))) rm(TB.qnt2.ratio )
if (!is.na(match("CR.qnt1.ratio",obj))) rm(CR.qnt1.ratio )
if (!is.na(match("CR.qnt2.ratio",obj))) rm(CR.qnt2.ratio )

if (!is.na(match("p.TB.init",obj)))      rm(p.TB.init)
if (!is.na(match("TB.qnt1.init",obj)))   rm(TB.qnt1.init)
if (!is.na(match("TB.qnt2.init",obj)))   rm(TB.qnt2.init)
if (!is.na(match("CR.qnt1.init",obj)))   rm(CR.qnt1.init)
if (!is.na(match("CR.qnt2.init",obj)))   rm(CR.qnt2.init)

if (!is.na(match("p.TB.int",obj)))      rm(p.TB.int)
if (!is.na(match("TB.qnt1.int",obj)))   rm(TB.qnt1.int)
if (!is.na(match("TB.qnt2.int",obj)))   rm(TB.qnt2.int)
if (!is.na(match("CR.qnt1.int",obj)))   rm(CR.qnt1.int)
if (!is.na(match("CR.qnt2.int",obj)))   rm(CR.qnt2.int)

if (!is.na(match("qnt1TB",obj))) rm(qnt1TB)
if (!is.na(match("qnt2TB",obj))) rm(qnt2TB)
if (!is.na(match("qnt1CR",obj))) rm(qnt1CR)
if (!is.na(match("qnt2CR",obj))) rm(qnt2CR)

if (!is.na(match("MS.run",obj))) rm(MS.run)
if (!is.na(match("MS.results",obj))) rm(MS.results)
if (!is.na(match("MS.check",obj))) rm(MS.check)
if (!is.na(match("MS.graph",obj))) rm(MS.graph)
if (!is.na(match("MS.size",obj))) rm(MS.size)
if (!is.na(match("MS.graph.page",obj)))  rm(MS.graph.page)
if (!is.na(match("MS.screen",obj)))  rm(MS.screen)
if (!is.na(match("MS.InitialValues",obj)))  rm(MS.InitialValues)
if (!is.na(match("RndSeed",obj)))  rm(RndSeed)

if (!is.na(match("ind.names",obj))) rm(ind.names)
if (!is.na(match("group.names",obj))) rm(group.names)
if (!is.na(match("outfile",obj))) rm(outfile)
if (!is.na(match("infile",obj))) rm(infile)
if (!is.na(match("freq.column",obj))) rm(freq.column)
if (!is.na(match("distTB",obj))) rm(distTB)
if (!is.na(match("distCR",obj))) rm(distCR)
if (!is.na(match("header",obj))) rm(header)
